/**
 * classe SauveVoiture (Application)
 * saisie interactive d'une voiture et sauvegarde dans un fichier
 * @author Matthias Colin
 * @version 1.0 (9/06/2009)
 */

import java.awt.Color;
import java.io.*;

public class SauveVoiture {


	/**
	 * lire une information au clavier avec un message d'attente
	 * @param messageInfo le message d'attente (prompt)
	 * @return chaîne lu au clavier
	 * @throws IOError en cas d'erreur grave d'entrée/sortie
	 */
    private static String lireInfo(String messageInfo) throws IOError {
		String infoLue;
		System.out.print(messageInfo + " : ");
		// lecture clavier et nettoyage des espaces en tête ou en queue de
		// chaîne
		infoLue = System.console().readLine().trim();
		return infoLue;
	}

	/**
	 * lire un entier au clavier jusqu'à ce qu'il soit dans l'intervalle
	 * demandé
	 * @param messageInfo le message d'attente (prompt)
	 * @param borneMin choix minimal
	 * @param borneMax choix maximal
	 * @return choix numérique lu au clavier
	 * @throws IOError en cas d'erreur grave d'entrée/sortie
	 */
	private static int lireEntier(String messageInfo, int borneMin, 
			int borneMax) throws IOError {
		int choix = -1;
		boolean erreur;
		String lecture; // buffer de lecture clavier

		// lecture du choix jusqu'à ce qu'elle soit correcte
		do {
			try {
				erreur = false;
				lecture = lireInfo(messageInfo + " ["+borneMin + "-" + borneMax
						+"]");
				choix = Integer.parseInt(lecture);
			} catch (NumberFormatException e) {
				// ce n'est pas un entier qui a été lu au clavier
				erreur = true;
			}
		} while (erreur || choix<borneMin || choix>borneMax);
		return choix;
	}

	/**
	 * saisie d'une Voiture au clavier
	 * @return une nouvelle voiture suivant les propriétés saisies
	 * @throws SaisieVoitureException en cas de problème de saisie clavier non
	 * résolu
	 */
	private static Voiture saisieVoiture() throws SaisieVoitureException {
		// propriétés de la voiture
		String modele, marque;
		int couleurR, couleurG, couleurB, couleurA; // composantes couleur
		// Voiture à sauvegarder
		Voiture voiture;
	
		try {
			System.out.println("Saisie d'une nouvelle voiture");
			couleurR = lireEntier("couleur : composante rouge", 0,255);
			couleurG = lireEntier("couleur : composante verte", 0,255);
			couleurB = lireEntier("couleur : composante bleue", 0,255);
			couleurA = lireEntier("couleur : composante alpha", 0,255);
			marque = lireInfo("marque");
			modele = lireInfo("modele");
			voiture = new Voiture(new Color(couleurR, couleurG, couleurB,
					couleurA), marque, modele);
			return voiture;
		} catch (IOError e) {
			throw (new SaisieVoitureException("Problème grave d'entrée/sortie",
					e));
		}
	}

	/**
	 * sauvegarde d'une Voiture dans un fichier
	 * @param nomFichier le nom du fichier contenant la sauvegarde
	 * @throws IOException (et ses dérivées) en cas de problème de sauvegarde
	 */
	private static void sauverVoiture(String nomFichier, Voiture voiture,
			boolean append) throws IOException {
		ObjectOutputStream out;
		out = new ObjectOutputStream(new FileOutputStream(nomFichier, append));
		out.writeObject(voiture);
		out.close();
	}
			
	public static void main(String[] args) {
		// Voitures à sauvegarder
		Voiture voiture1, voiture2;
		// nom du fichier de sauvegarde
		String nomFichier = "voitures.obj";

		try {
			voiture1 = saisieVoiture();
			voiture2 = saisieVoiture();
			sauverVoiture(nomFichier, voiture1, false);
			sauverVoiture(nomFichier, voiture1, true);
		} catch (SaisieVoitureException e) {
			System.err.println("Problème lors de la saisie de la voiture : " 
					+ e.getMessage());
		} catch (FileNotFoundException e) {
			System.err.println("Le fichier " + nomFichier + 
					" n'existe pas ou ne peut être écrit : " + e.getMessage());
		} catch (IOException e) {
			System.err.println("Problème lors de la sauvegarde des voitures : "
				 	+ e.getMessage());
		}
	}
}
